@@ -20,7 +20,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-__version__ = '2.1.2'
+__version__ = '2.2.0'
 __author__ = 'Raphaël Barrois <raphael.barrois+fboy@polytechnique.org>'
 
 
@@ -38,6 +38,7 @@ from .base import (
     use_strategy,
 )
 
+# Backward compatibility; this should be removed soon.
 from .mogo import MogoFactory
 from .django import DjangoModelFactory
 
@@ -57,6 +58,8 @@ from .declarations import (
 )
 
 from .helpers import (
+    debug,
+
     build,
     create,
     stub,
