@@ -21,11 +21,15 @@
 # THE SOFTWARE.
 
 
-import collections
 import itertools
 import warnings
+import logging
 
-from factory import utils
+from . import compat
+from . import utils
+
+
+logger = logging.getLogger('factory.generate')
 
 
 class OrderedDeclaration(object):
@@ -36,7 +40,7 @@ class OrderedDeclaration(object):
     in the same factory.
     """
 
-    def evaluate(self, sequence, obj, containers=()):
+    def evaluate(self, sequence, obj, create, extra=None, containers=()):
         """Evaluate this declaration.
 
         Args:
@@ -46,6 +50,10 @@ class OrderedDeclaration(object):
                 attributes
             containers (list of containers.LazyStub): The chain of SubFactory
                 which led to building this object.
+            create (bool): whether the target class should be 'built' or
+                'created'
+            extra (DeclarationDict or None): extracted key/value extracted from
+                the attribute prefix
         """
         raise NotImplementedError('This is an abstract method')
 
@@ -62,7 +70,8 @@ class LazyAttribute(OrderedDeclaration):
         super(LazyAttribute, self).__init__(*args, **kwargs)
         self.function = function
 
-    def evaluate(self, sequence, obj, containers=()):
+    def evaluate(self, sequence, obj, create, extra=None, containers=()):
+        logger.debug("LazyAttribute: Evaluating %r on %r", self.function, obj)
         return self.function(obj)
 
 
@@ -121,14 +130,23 @@ class SelfAttribute(OrderedDeclaration):
         self.attribute_name = attribute_name
         self.default = default
 
-    def evaluate(self, sequence, obj, containers=()):
+    def evaluate(self, sequence, obj, create, extra=None, containers=()):
         if self.depth > 1:
             # Fetching from a parent
             target = containers[self.depth - 2]
         else:
             target = obj
+
+        logger.debug("SelfAttribute: Picking attribute %r on %r", self.attribute_name, target)
         return deepgetattr(target, self.attribute_name, self.default)
 
+    def __repr__(self):
+        return '<%s(%r, default=%r)>' % (
+            self.__class__.__name__,
+            self.attribute_name,
+            self.default,
+        )
+
 
 class Iterator(OrderedDeclaration):
     """Fill this value using the values returned by an iterator.
@@ -145,30 +163,19 @@ class Iterator(OrderedDeclaration):
         self.getter = getter
 
         if cycle:
-            self.iterator = itertools.cycle(iterator)
-        else:
-            self.iterator = iter(iterator)
+            iterator = itertools.cycle(iterator)
+        self.iterator = utils.ResetableIterator(iterator)
 
-    def evaluate(self, sequence, obj, containers=()):
-        value = next(self.iterator)
+    def evaluate(self, sequence, obj, create, extra=None, containers=()):
+        logger.debug("Iterator: Fetching next value from %r", self.iterator)
+        value = next(iter(self.iterator))
         if self.getter is None:
             return value
         return self.getter(value)
 
-
-class InfiniteIterator(Iterator):
-    """Same as Iterator, but make the iterator infinite by cycling at the end.
-
-    Attributes:
-        iterator (iterable): the iterator, once made infinite.
-    """
-
-    def __init__(self, iterator):
-        warnings.warn(
-            "factory.InfiniteIterator is deprecated, and will be removed in the "
-            "future. Please use factory.Iterator instead.",
-            PendingDeprecationWarning, 2)
-        return super(InfiniteIterator, self).__init__(iterator, cycle=True)
+    def reset(self):
+        """Reset the internal iterator."""
+        self.iterator.reset()
 
 
 class Sequence(OrderedDeclaration):
@@ -182,12 +189,13 @@ class Sequence(OrderedDeclaration):
         type (function): A function converting an integer into the expected kind
             of counter for the 'function' attribute.
     """
-    def __init__(self, function, type=str):
+    def __init__(self, function, type=int):  # pylint: disable=W0622
         super(Sequence, self).__init__()
         self.function = function
         self.type = type
 
-    def evaluate(self, sequence, obj, containers=()):
+    def evaluate(self, sequence, obj, create, extra=None, containers=()):
+        logger.debug("Sequence: Computing next value of %r for seq=%d", self.function, sequence)
         return self.function(self.type(sequence))
 
 
@@ -200,7 +208,9 @@ class LazyAttributeSequence(Sequence):
         type (function): A function converting an integer into the expected kind
             of counter for the 'function' attribute.
     """
-    def evaluate(self, sequence, obj, containers=()):
+    def evaluate(self, sequence, obj, create, extra=None, containers=()):
+        logger.debug("LazyAttributeSequence: Computing next value of %r for seq=%d, obj=%r",
+                self.function, sequence, obj)
         return self.function(obj, self.type(sequence))
 
 
@@ -218,7 +228,7 @@ class ContainerAttribute(OrderedDeclaration):
         self.function = function
         self.strict = strict
 
-    def evaluate(self, sequence, obj, containers=()):
+    def evaluate(self, sequence, obj, create, extra=None, containers=()):
         """Evaluate the current ContainerAttribute.
 
         Args:
@@ -251,11 +261,20 @@ class ParameteredAttribute(OrderedDeclaration):
 
     CONTAINERS_FIELD = '__containers'
 
+    # Whether to add the current object to the stack of containers
+    EXTEND_CONTAINERS = False
+
     def __init__(self, **kwargs):
         super(ParameteredAttribute, self).__init__()
         self.defaults = kwargs
 
-    def evaluate(self, create, extra, containers):
+    def _prepare_containers(self, obj, containers=()):
+        if self.EXTEND_CONTAINERS:
+            return (obj,) + tuple(containers)
+
+        return containers
+
+    def evaluate(self, sequence, obj, create, extra=None, containers=()):
         """Evaluate the current definition and fill its attributes.
 
         Uses attributes definition in the following order:
@@ -274,14 +293,17 @@ class ParameteredAttribute(OrderedDeclaration):
         if extra:
             defaults.update(extra)
         if self.CONTAINERS_FIELD:
+            containers = self._prepare_containers(obj, containers)
             defaults[self.CONTAINERS_FIELD] = containers
 
-        return self.generate(create, defaults)
+        return self.generate(sequence, obj, create, defaults)
 
-    def generate(self, create, params):  # pragma: no cover
+    def generate(self, sequence, obj, create, params):  # pragma: no cover
         """Actually generate the related attribute.
 
         Args:
+            sequence (int): the current sequence number
+            obj (LazyStub): the object being constructed
             create (bool): whether the calling factory was in 'create' or
                 'build' mode
             params (dict): parameters inherited from init and evaluation-time
@@ -293,6 +315,40 @@ class ParameteredAttribute(OrderedDeclaration):
         raise NotImplementedError()
 
 
+class _FactoryWrapper(object):
+    """Handle a 'factory' arg.
+
+    Such args can be either a Factory subclass, or a fully qualified import
+    path for that subclass (e.g 'myapp.factories.MyFactory').
+    """
+    def __init__(self, factory_or_path):
+        self.factory = None
+        self.module = self.name = ''
+        if isinstance(factory_or_path, type):
+            self.factory = factory_or_path
+        else:
+            if not (compat.is_string(factory_or_path) and '.' in factory_or_path):
+                raise ValueError(
+                        "A factory= argument must receive either a class "
+                        "or the fully qualified path to a Factory subclass; got "
+                        "%r instead." % factory_or_path)
+            self.module, self.name = factory_or_path.rsplit('.', 1)
+
+    def get(self):
+        if self.factory is None:
+            self.factory = utils.import_object(
+                self.module,
+                self.name,
+            )
+        return self.factory
+
+    def __repr__(self):
+        if self.factory is None:
+            return '<_FactoryImport: %s.%s>' % (self.module, self.name)
+        else:
+            return '<_FactoryImport: %s>' % self.factory.__class__
+
+
 class SubFactory(ParameteredAttribute):
     """Base class for attributes based upon a sub-factory.
 
@@ -302,29 +358,17 @@ class SubFactory(ParameteredAttribute):
         factory (base.Factory): the wrapped factory
     """
 
+    EXTEND_CONTAINERS = True
+
     def __init__(self, factory, **kwargs):
         super(SubFactory, self).__init__(**kwargs)
-        if isinstance(factory, type):
-            self.factory = factory
-            self.factory_module = self.factory_name = ''
-        else:
-            # Must be a string
-            if not isinstance(factory, basestring) or '.' not in factory:
-                raise ValueError(
-                        "The argument of a SubFactory must be either a class "
-                        "or the fully qualified path to a Factory class; got "
-                        "%r instead." % factory)
-            self.factory = None
-            self.factory_module, self.factory_name = factory.rsplit('.', 1)
+        self.factory_wrapper = _FactoryWrapper(factory)
 
     def get_factory(self):
         """Retrieve the wrapped factory.Factory subclass."""
-        if self.factory is None:
-            # Must be a module path
-            self.factory = utils.import_object(self.factory_module, self.factory_name)
-        return self.factory
+        return self.factory_wrapper.get()
 
-    def generate(self, create, params):
+    def generate(self, sequence, obj, create, params):
         """Evaluate the current definition and fill its attributes.
 
         Args:
@@ -334,42 +378,63 @@ class SubFactory(ParameteredAttribute):
                 override the wrapped factory's defaults
         """
         subfactory = self.get_factory()
-        if create:
-            return subfactory.create(**params)
-        else:
-            return subfactory.build(**params)
+        logger.debug("SubFactory: Instantiating %s.%s(%s), create=%r",
+            subfactory.__module__, subfactory.__name__,
+            utils.log_pprint(kwargs=params),
+            create,
+        )
+        return subfactory.simple_generate(create, **params)
 
 
-class CircularSubFactory(SubFactory):
-    """Use to solve circular dependencies issues."""
-    def __init__(self, module_name, factory_name, **kwargs):
-        factory = '%s.%s' % (module_name, factory_name)
-        warnings.warn(
-            "factory.CircularSubFactory is deprecated and will be removed in "
-            "the future. "
-            "Please replace factory.CircularSubFactory('module', 'symbol') "
-            "with factory.SubFactory('module.symbol').",
-            PendingDeprecationWarning, 2)
+class Dict(SubFactory):
+    """Fill a dict with usual declarations."""
 
-        super(CircularSubFactory, self).__init__(factory, **kwargs)
+    def __init__(self, params, dict_factory='factory.DictFactory'):
+        super(Dict, self).__init__(dict_factory, **dict(params))
+
+    def generate(self, sequence, obj, create, params):
+        dict_factory = self.get_factory()
+        logger.debug("Dict: Building dict(%s)", utils.log_pprint(kwargs=params))
+        return dict_factory.simple_generate(create,
+            __sequence=sequence,
+            **params)
+
+
+class List(SubFactory):
+    """Fill a list with standard declarations."""
+
+    def __init__(self, params, list_factory='factory.ListFactory'):
+        params = dict((str(i), v) for i, v in enumerate(params))
+        super(List, self).__init__(list_factory, **params)
+
+    def generate(self, sequence, obj, create, params):
+        list_factory = self.get_factory()
+        logger.debug('List: Building list(%s)',
+            utils.log_pprint(args=[v for _i, v in sorted(params.items())]),
+        )
+        return list_factory.simple_generate(create,
+            __sequence=sequence,
+            **params)
+
+
+class ExtractionContext(object):
+    """Private class holding all required context from extraction to postgen."""
+    def __init__(self, value=None, did_extract=False, extra=None, for_field=''):
+        self.value = value
+        self.did_extract = did_extract
+        self.extra = extra or {}
+        self.for_field = for_field
+
+    def __repr__(self):
+        return 'ExtractionContext(%r, %r, %r)' % (
+            self.value,
+            self.did_extract,
+            self.extra,
+        )
 
 
 class PostGenerationDeclaration(object):
-    """Declarations to be called once the target object has been generated.
-
-    Attributes:
-        extract_prefix (str): prefix to use when extracting attributes from
-            the factory's declaration for this declaration. If empty, uses
-            the attribute name of the PostGenerationDeclaration.
-    """
-
-    def __init__(self, extract_prefix=None):
-        if extract_prefix:
-            warnings.warn(
-                "The extract_prefix argument to PostGeneration declarations "
-                "is deprecated and will be removed in the future.",
-                PendingDeprecationWarning, 3)
-        self.extract_prefix = extract_prefix
+    """Declarations to be called once the target object has been generated."""
 
     def extract(self, name, attrs):
         """Extract relevant attributes from a dict.
@@ -384,54 +449,45 @@ class PostGenerationDeclaration(object):
             (object, dict): a tuple containing the attribute at 'name' (if
                 provided) and a dict of extracted attributes
         """
-        if self.extract_prefix:
-            extract_prefix = self.extract_prefix
-        else:
-            extract_prefix = name
-        extracted = attrs.pop(extract_prefix, None)
-        kwargs = utils.extract_dict(extract_prefix, attrs)
-        return extracted, kwargs
+        try:
+            extracted = attrs.pop(name)
+            did_extract = True
+        except KeyError:
+            extracted = None
+            did_extract = False
 
-    def call(self, obj, create, extracted=None, **kwargs):  # pragma: no cover
+        kwargs = utils.extract_dict(name, attrs)
+        return ExtractionContext(extracted, did_extract, kwargs, name)
+
+    def call(self, obj, create, extraction_context):  # pragma: no cover
         """Call this hook; no return value is expected.
 
         Args:
             obj (object): the newly generated object
             create (bool): whether the object was 'built' or 'created'
-            extracted (object): the value given for <extract_prefix> in the
-                object definition, or None if not provided.
-            kwargs (dict): declarations extracted from the object
-                definition for this hook
+            extraction_context: An ExtractionContext containing values
+                extracted from the containing factory's declaration
         """
         raise NotImplementedError()
 
 
 class PostGeneration(PostGenerationDeclaration):
     """Calls a given function once the object has been generated."""
-    def __init__(self, function, extract_prefix=None):
-        super(PostGeneration, self).__init__(extract_prefix)
+    def __init__(self, function):
+        super(PostGeneration, self).__init__()
         self.function = function
 
-    def call(self, obj, create, extracted=None, **kwargs):
-        return self.function(obj, create, extracted, **kwargs)
-
-
-def post_generation(*args, **kwargs):
-    assert len(args) + len(kwargs) <= 1, "post_generation takes at most one argument."
-    if args and callable(args[0]):
-        # Called as @post_generation applied to a function
-        return PostGeneration(args[0])
-    else:
-        warnings.warn(
-            "The @post_generation should now be applied directly to the "
-            "function, without parameters. The @post_generation() and "
-            "@post_generation(extract_prefix='xxx') syntaxes are deprecated "
-            "and will be removed in the future; use @post_generation instead.",
-            PendingDeprecationWarning, 2)
-        extract_prefix = kwargs.get('extract_prefix')
-        def decorator(fun):
-            return PostGeneration(fun, extract_prefix=extract_prefix)
-        return decorator
+    def call(self, obj, create, extraction_context):
+        logger.debug('PostGeneration: Calling %s.%s(%s)',
+            self.function.__module__,
+            self.function.__name__,
+            utils.log_pprint(
+                (obj, create, extraction_context.value),
+                extraction_context.extra,
+            ),
+        )
+        return self.function(obj, create,
+            extraction_context.value, **extraction_context.extra)
 
 
 class RelatedFactory(PostGenerationDeclaration):
@@ -444,39 +500,48 @@ class RelatedFactory(PostGenerationDeclaration):
             calling the related factory
     """
 
-    def __init__(self, factory, name='', **defaults):
-        super(RelatedFactory, self).__init__(extract_prefix=None)
-        self.name = name
-        self.defaults = defaults
+    def __init__(self, factory, factory_related_name='', **defaults):
+        super(RelatedFactory, self).__init__()
+        if factory_related_name == '' and defaults.get('name') is not None:
+            warnings.warn(
+                "Usage of RelatedFactory(SomeFactory, name='foo') is deprecated"
+                " and will be removed in the future. Please use the"
+                " RelatedFactory(SomeFactory, 'foo') or"
+                " RelatedFactory(SomeFactory, factory_related_name='foo')"
+                " syntax instead", PendingDeprecationWarning, 2)
+            factory_related_name = defaults.pop('name')
 
-        if isinstance(factory, type):
-            self.factory = factory
-            self.factory_module = self.factory_name = ''
-        else:
-            # Must be a string
-            if not isinstance(factory, basestring) or '.' not in factory:
-                raise ValueError(
-                        "The argument of a SubFactory must be either a class "
-                        "or the fully qualified path to a Factory class; got "
-                        "%r instead." % factory)
-            self.factory = None
-            self.factory_module, self.factory_name = factory.rsplit('.', 1)
+        self.name = factory_related_name
+        self.defaults = defaults
+        self.factory_wrapper = _FactoryWrapper(factory)
 
     def get_factory(self):
         """Retrieve the wrapped factory.Factory subclass."""
-        if self.factory is None:
-            # Must be a module path
-            self.factory = utils.import_object(self.factory_module, self.factory_name)
-        return self.factory
+        return self.factory_wrapper.get()
+
+    def call(self, obj, create, extraction_context):
+        factory = self.get_factory()
+
+        if extraction_context.did_extract:
+            # The user passed in a custom value
+            logger.debug('RelatedFactory: Using provided %r instead of '
+                    'generating %s.%s.',
+                    extraction_context.value,
+                    factory.__module__, factory.__name__,
+            )
+            return extraction_context.value
 
-    def call(self, obj, create, extracted=None, **kwargs):
         passed_kwargs = dict(self.defaults)
-        passed_kwargs.update(kwargs)
+        passed_kwargs.update(extraction_context.extra)
         if self.name:
             passed_kwargs[self.name] = obj
 
-        factory = self.get_factory()
-        factory.simple_generate(create, **passed_kwargs)
+        logger.debug('RelatedFactory: Generating %s.%s(%s)',
+            factory.__module__,
+            factory.__name__,
+            utils.log_pprint((create,), passed_kwargs),
+        )
+        return factory.simple_generate(create, **passed_kwargs)
 
 
 class PostGenerationMethodCall(PostGenerationDeclaration):
@@ -490,53 +555,30 @@ class PostGenerationMethodCall(PostGenerationDeclaration):
     Example:
         class UserFactory(factory.Factory):
             ...
-            password = factory.PostGenerationMethodCall('set_password', password='')
+            password = factory.PostGenerationMethodCall('set_pass', password='')
     """
     def __init__(self, method_name, *args, **kwargs):
-        extract_prefix = kwargs.pop('extract_prefix', None)
-        super(PostGenerationMethodCall, self).__init__(extract_prefix)
+        super(PostGenerationMethodCall, self).__init__()
         self.method_name = method_name
         self.method_args = args
         self.method_kwargs = kwargs
 
-    def call(self, obj, create, extracted=None, **kwargs):
-        if extracted is None:
+    def call(self, obj, create, extraction_context):
+        if not extraction_context.did_extract:
             passed_args = self.method_args
 
         elif len(self.method_args) <= 1:
             # Max one argument expected
-            passed_args = (extracted,)
+            passed_args = (extraction_context.value,)
         else:
-            passed_args = tuple(extracted)
+            passed_args = tuple(extraction_context.value)
 
         passed_kwargs = dict(self.method_kwargs)
-        passed_kwargs.update(kwargs)
+        passed_kwargs.update(extraction_context.extra)
         method = getattr(obj, self.method_name)
-        method(*passed_args, **passed_kwargs)
-
-
-# Decorators... in case lambdas don't cut it
-
-def lazy_attribute(func):
-    return LazyAttribute(func)
-
-def iterator(func):
-    """Turn a generator function into an iterator attribute."""
-    return Iterator(func())
-
-def infinite_iterator(func):
-    """Turn a generator function into an infinite iterator attribute."""
-    warnings.warn(
-        "@factory.infinite_iterator is deprecated and will be removed in the "
-        "future. Please use @factory.iterator instead.",
-        PendingDeprecationWarning, 2)
-    return InfiniteIterator(func())
-
-def sequence(func):
-    return Sequence(func)
-
-def lazy_attribute_sequence(func):
-    return LazyAttributeSequence(func)
-
-def container_attribute(func):
-    return ContainerAttribute(func, strict=False)
+        logger.debug('PostGenerationMethodCall: Calling %r.%s(%s)',
+            obj,
+            self.method_name,
+            utils.log_pprint(passed_args, passed_kwargs),
+        )
+        return method(*passed_args, **passed_kwargs)
