@@ -68,6 +68,16 @@ class LazyStubTestCase(unittest.TestCase):
             containers=())
         self.assertEqual(2, stub.one)
 
+    def test_access_parent(self):
+        """Test simple access to a stub' parent."""
+        o1 = containers.LazyStub({'rank': 1})
+        o2 = containers.LazyStub({'rank': 2}, (o1,))
+        stub = containers.LazyStub({'rank': 3}, (o2, o1))
+
+        self.assertEqual(3, stub.rank)
+        self.assertEqual(2, stub.factory_parent.rank)
+        self.assertEqual(1, stub.factory_parent.factory_parent.rank)
+
     def test_cyclic_definition(self):
         class LazyAttr(containers.LazyValue):
             def __init__(self, attrname):
@@ -253,7 +263,7 @@ class AttributeBuilderTestCase(unittest.TestCase):
         self.assertEqual({'one': 2}, ab.build(create=False))
 
     def test_factory_defined_sequence(self):
-        seq = declarations.Sequence(lambda n: 'xx' + n)
+        seq = declarations.Sequence(lambda n: 'xx%d' % n)
 
         class FakeFactory(object):
             @classmethod
@@ -270,7 +280,7 @@ class AttributeBuilderTestCase(unittest.TestCase):
         self.assertEqual({'one': 'xx1'}, ab.build(create=False))
 
     def test_additionnal_sequence(self):
-        seq = declarations.Sequence(lambda n: 'xx' + n)
+        seq = declarations.Sequence(lambda n: 'xx%d' % n)
 
         class FakeFactory(object):
             @classmethod
@@ -287,8 +297,8 @@ class AttributeBuilderTestCase(unittest.TestCase):
         self.assertEqual({'one': 1, 'two': 'xx1'}, ab.build(create=False))
 
     def test_replaced_sequence(self):
-        seq = declarations.Sequence(lambda n: 'xx' + n)
-        seq2 = declarations.Sequence(lambda n: 'yy' + n)
+        seq = declarations.Sequence(lambda n: 'xx%d' % n)
+        seq2 = declarations.Sequence(lambda n: 'yy%d' % n)
 
         class FakeFactory(object):
             @classmethod
@@ -349,5 +359,5 @@ class AttributeBuilderTestCase(unittest.TestCase):
         pass
 
 
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: no cover
     unittest.main()
